@@ -1,6 +1,6 @@
 /* hexter DSSI software synthesizer plugin
  *
- * Copyright (C) 2004-2006 Sean Bolton and others.
+ * Copyright (C) 2004-2007 Sean Bolton and others.
  *
  * Portions of this file may have come from Juan Linietsky's
  * rx-saturno, copyright (C) 2002 by Juan Linietsky.
@@ -93,16 +93,19 @@ dx7_voice_note_on(hexter_instance_t *instance, dx7_voice_t *voice,
         DEBUG_MESSAGE(DB_NOTE, " dx7_voice_note_on in polyphonic/new section: key %d, mono %d, old status %d\n", key, instance->monophonic, voice->status);
 
         dx7_voice_setup_note(instance, voice);
-        dx7_voice_update_pressure_mod(instance, voice);
 
     } else {
 
         /* synth is monophonic, and we're modifying a playing voice */
         DEBUG_MESSAGE(DB_NOTE, " dx7_voice_note_on in monophonic section: old key %d => new key %d\n", instance->held_keys[0], key);
 
+        /* retrigger LFO if needed */
+        dx7_lfo_set(instance, voice);
+
         /* set new pitch */
+        voice->mods_serial = instance->mods_serial - 1;
+        /* -FIX- dx7_portamento_prepare(instance, voice); */
         dx7_voice_recalculate_freq_and_inc(instance, voice);
-        dx7_voice_update_pressure_mod(instance, voice);
 
         /* if in 'on' or 'both' modes, and key has changed, then re-trigger EGs */
         if ((instance->monophonic == DSSP_MONO_MODE_ON ||
@@ -114,6 +117,8 @@ dx7_voice_note_on(hexter_instance_t *instance, dx7_voice_t *voice,
         /* all other variables stay what they are */
     }
 
+    instance->last_key = key;
+
     if (instance->monophonic) {
 
         /* add new key to the list of held keys */
@@ -166,8 +171,9 @@ dx7_voice_note_off(hexter_instance_t *instance, dx7_voice_t *voice,
                 /* most-recently-played key has changed */
                 voice->key = instance->held_keys[0];
                 DEBUG_MESSAGE(DB_NOTE, " note-off in monophonic section: changing pitch to %d\n", voice->key);
+                voice->mods_serial = instance->mods_serial - 1;
+                /* -FIX- dx7_portamento_prepare(instance, voice); */
                 dx7_voice_recalculate_freq_and_inc(instance, voice);
-                dx7_voice_update_pressure_mod(instance, voice);
 
                 /* if mono mode is 'both', re-trigger EGs */
                 if (instance->monophonic == DSSP_MONO_MODE_BOTH && !_RELEASED(voice)) {
@@ -264,7 +270,7 @@ dx7_op_eg_set_increment(hexter_instance_t *instance, dx7_op_eg_t *eg,
             } else {
                 /* these are the exploited delays */
                 need_compensation = 0;
-                /* !FIX! this doesn't make WATER GDN work? */
+                /* -FIX- this doesn't make WATER GDN work? */
                 duration = dx7_voice_eg_rate_rise_duration[new_rate] *
                            (float)(new_level - current_level) / 100.0f;
             }
@@ -402,6 +408,7 @@ dx7_op_eg_set_phase(hexter_instance_t *instance, dx7_op_eg_t *eg, int phase)
             dx7_op_eg_set_increment(instance, eg, eg->rate[phase], eg->level[phase]);
             if (eg->duration == 1 && eg->increment == 0)
                 dx7_op_eg_set_next_phase(instance, eg);
+
         }
     } else {
 
@@ -411,6 +418,7 @@ dx7_op_eg_set_phase(hexter_instance_t *instance, dx7_op_eg_t *eg, int phase)
             dx7_op_eg_set_increment(instance, eg, eg->rate[phase], eg->level[phase]);
             if (eg->duration == 1 && eg->increment == 0)
                 dx7_op_eg_set_next_phase(instance, eg);
+
         }
     }
 }
@@ -494,14 +502,14 @@ dx7_op_envelope_prepare(hexter_instance_t *instance, dx7_op_t *op,
     /* Things which affect this calculation: transpose, ? */
     /* rate_bump = lrintf((float)op->rate_scaling * (float)(transposed_note - 21) / (126.0f - 21.0f) * 127.0f / 128.0f * 6.0f - 0.5f); */
     rate_bump = lrintf((float)op->rate_scaling * (float)(transposed_note - 21) / (126.0f - 21.0f) * 127.0f / 128.0f * 6.5f - 0.5f);
-    /* !FIX! just a hunch: try it again with "* 6.0f" but also "(120.0f - 21.0f)" instead of "(126.0f - 21.0f)": */
+    /* -FIX- just a hunch: try it again with "* 6.0f" but also "(120.0f - 21.0f)" instead of "(126.0f - 21.0f)": */
     /* rate_bump = lrintf((float)op->rate_scaling * (float)(transposed_note - 21) / (120.0f - 21.0f) * 127.0f / 128.0f * 6.0f - 0.5f); */
 
     for (i=0;i<4;i++) {
 
         float level = (float)op->eg.base_level[i];
 
-        /* !FIX! is this scaling of eg.base_level values to og.level values correct, i.e. does a softer
+        /* -FIX- is this scaling of eg.base_level values to og.level values correct, i.e. does a softer
          * velocity shorten the time, since the rate stays the same? */
         level = level * (float)scaled_output_level / 99.0f + vel_adj;
         if (level < 0.0f)
@@ -513,6 +521,7 @@ dx7_op_envelope_prepare(hexter_instance_t *instance, dx7_op_t *op,
 
         op->eg.rate[i] = op->eg.base_rate[i] + rate_bump;
         if (op->eg.rate[i] > 99) op->eg.rate[i] = 99;
+
     }
 
     op->eg.value = INT_TO_FP(op->eg.level[3]);
@@ -520,6 +529,21 @@ dx7_op_envelope_prepare(hexter_instance_t *instance, dx7_op_t *op,
     dx7_op_eg_set_phase(instance, &op->eg, 0);
 }
 
+void
+dx7_eg_init_constants(hexter_instance_t *instance)
+{
+    float duration = dx7_voice_eg_rate_rise_duration[99] *
+                     (dx7_voice_eg_rate_rise_percent[99] -
+                      dx7_voice_eg_rate_rise_percent[0]);
+
+    instance->dx7_eg_max_slew = lrintf((float)INT_TO_FP(99) /
+                                       (duration * instance->sample_rate));
+
+    instance->nugget_rate = instance->sample_rate / (float)HEXTER_NUGGET_SIZE;
+
+    instance->ramp_duration = lrintf(instance->sample_rate * 0.006f);  /* 6ms ramp */
+}
+
 /* ===== pitch envelope functions ===== */
 
 /*
@@ -534,7 +558,7 @@ dx7_pitch_eg_set_increment(hexter_instance_t *instance, dx7_pitch_eg_t *eg,
     /* translate 0-99 level to shift in semitones */
     eg->target = dx7_voice_pitch_level_to_shift[new_level];
 
-    /* !FIX! This is just a quick approximation that I derived from
+    /* -FIX- This is just a quick approximation that I derived from
      * regression of Godric Wilkie's pitch eg timings. In particular,
      * it's not accurate for very slow envelopes. */
     duration = exp(((double)new_rate - 70.337897) / -25.580953) *
@@ -623,22 +647,42 @@ dx7_pitch_envelope_prepare(hexter_instance_t *instance, dx7_voice_t *voice)
     dx7_pitch_eg_set_phase(instance, &voice->pitch_eg, 0);
 }
 
+/* ===== portamento functions ===== */
+
 void
-dx7_eg_init_constants(hexter_instance_t *instance)
+dx7_portamento_set_segment(hexter_instance_t *instance, dx7_portamento_t *port)
 {
-    float duration = dx7_voice_eg_rate_rise_duration[99] *
-                     (dx7_voice_eg_rate_rise_percent[99] -
-                      dx7_voice_eg_rate_rise_percent[0]);
+    /* -FIX- implement portamento multi-segment curve */
+    port->increment = (port->target - port->value) / (double)port->duration;
+}
 
-    instance->dx7_eg_max_slew = lrintf((float)INT_TO_FP(99) /
-                                       (duration * instance->sample_rate));
+void
+dx7_portamento_prepare(hexter_instance_t *instance, dx7_voice_t *voice)
+{
+    dx7_portamento_t *port = &voice->portamento;
 
-    instance->nugget_rate = instance->sample_rate / (float)HEXTER_NUGGET_SIZE;
+    if (instance->portamento_time == 0 ||
+        instance->last_key == voice->key) {
+
+        port->segment = 0;
+        port->value = 0.0;
+
+    } else {
+
+        /* -FIX- implement portamento time and multi-segment curve */
+        float t = expf((float)(instance->portamento_time - 99) / 15.0f) * 18.0f; /* not at all related to what a real DX7 does */
+        port->segment = 1;
+        port->value = (double)(instance->last_key - voice->key);
+        port->duration = lrintf(instance->nugget_rate * t);
+        port->target = 0.0;
+
+        dx7_portamento_set_segment(instance, port);
+    }
 }
 
 /* ===== frequency related functions ===== */
 
-static int
+static inline int
 limit_note(int note) {
     while (note < 0)   note += 12;
     while (note > 127) note -= 12;
@@ -653,15 +697,15 @@ dx7_op_recalculate_increment(hexter_instance_t *instance, dx7_op_t *op)
     if (op->osc_mode) { /* fixed frequency */
         /* pitch envelope does not affect this */
 
-        /* !FIX! convert this to a table lookup for speed? */
+        /* -FIX- convert this to a table lookup for speed? */
         freq = instance->fixed_freq_multiplier *
                    exp(M_LN10 * ((double)(op->coarse & 3) + (double)op->fine / 100.0));
-        /* !FIX! figure out what to do with detune */
+        /* -FIX- figure out what to do with detune */
 
     } else {
     
         freq = op->frequency;
-        freq += ((double)op->detune - 7.0) / 32.0; /* !FIX! is this correct? */
+        freq += ((double)op->detune - 7.0) / 32.0; /* -FIX- is this correct? */
         if (op->coarse) {
             freq = freq * (double)op->coarse;
         } else {
@@ -682,13 +726,17 @@ dx7_voice_recalculate_frequency(hexter_instance_t *instance, dx7_voice_t *voice)
 
     instance->fixed_freq_multiplier = *instance->tuning / 440.0;
 
-    freq = voice->pitch_eg.value + instance->pitch_bend;
+    freq = voice->pitch_eg.value + voice->portamento.value +
+           instance->pitch_bend -
+           instance->lfo_value_for_pitch *
+               (voice->pitch_mod_depth_pmd * FP_TO_DOUBLE(voice->lfo_delay_value) +
+                voice->pitch_mod_depth_mods);
 
     voice->last_pitch = freq;
 
     freq += (double)(limit_note(voice->key + voice->transpose - 24));
 
-    /* !FIX! this maybe could be optimized */
+    /* -FIX- this maybe could be optimized */
     /*       a lookup table of 8k values would give ~1.5 cent accuracy,
      *       but then would interpolating that be faster than exp()? */
     freq = *instance->tuning * exp((freq - 69.0) * M_LN2 / 12.0);
@@ -727,18 +775,398 @@ dx7_voice_recalculate_volume(hexter_instance_t *instance, dx7_voice_t *voice)
     f += (float)instance->cc_volume * 41.0f / 16256.0f;
     i = lrintf(f - 0.5f);
     f -= (float)i;
-    voice->volume_target = FP_TO_FLOAT(dx7_voice_eg_ol_to_amp[i] +
-                                       f * (dx7_voice_eg_ol_to_amp[i + 1] -
+    voice->volume_target = (FP_TO_FLOAT(dx7_voice_eg_ol_to_amp[i]) +
+                            f * FP_TO_FLOAT(dx7_voice_eg_ol_to_amp[i + 1] -
                                             dx7_voice_eg_ol_to_amp[i])) *
                            0.110384f / dx7_voice_carrier_count[voice->algorithm];
 
     if (voice->volume_value < 0.0f) { /* initial setup */
         voice->volume_value = voice->volume_target;
-        voice->volume_count = 0;
+        voice->volume_duration = 0;
     } else {
-        voice->volume_count = (unsigned long)lrintf(instance->sample_rate / 20.0f);  /* 50ms ramp */
-        voice->volume_delta = (voice->volume_target - voice->volume_value) /
-                                  (float)voice->volume_count;
+        voice->volume_duration = instance->ramp_duration;
+        voice->volume_increment = (voice->volume_target - voice->volume_value) /
+                                      (float)voice->volume_duration;
+    }
+}
+
+/* ===== LFO functions ===== */
+
+/* dx7_lfo_set_speed
+ *
+ * called by dx7_lfo_reset() and dx7_lfo_set() to set LFO speed and phase
+ */
+static inline void
+dx7_lfo_set_speed(hexter_instance_t *instance)
+{
+    uint32_t period = lrintf(instance->sample_rate /
+                             dx7_voice_lfo_frequency[instance->lfo_speed]);
+
+    switch (instance->lfo_wave) {
+      default:
+      case 0:  /* triangle */
+        instance->lfo_phase = 0;
+        instance->lfo_value = 0;
+        instance->lfo_duration0 = period / 2;
+        instance->lfo_duration1 = period - instance->lfo_duration0;
+        instance->lfo_increment0 = FP_SIZE / (int32_t)instance->lfo_duration0;
+        instance->lfo_increment1 = -instance->lfo_increment0;
+        instance->lfo_duration = instance->lfo_duration0;
+        instance->lfo_increment = instance->lfo_increment0;
+        break;
+      case 1:  /* saw down */
+        instance->lfo_phase = 0;
+        instance->lfo_value = 0;
+        if (period >= (instance->ramp_duration * 4)) {
+            instance->lfo_duration0 = period - instance->ramp_duration;
+            instance->lfo_duration1 = instance->ramp_duration;
+        } else {
+            instance->lfo_duration0 = period * 3 / 4;
+            instance->lfo_duration1 = period - instance->lfo_duration0;
+        }
+        instance->lfo_increment0 = FP_SIZE / (int32_t)instance->lfo_duration0;
+        instance->lfo_increment1 = -FP_SIZE / (int32_t)instance->lfo_duration1;
+        instance->lfo_duration = instance->lfo_duration0;
+        instance->lfo_increment = instance->lfo_increment0;
+        break;
+      case 2:  /* saw up */
+        instance->lfo_phase = 1;
+        instance->lfo_value = FP_SIZE;
+        if (period >= (instance->ramp_duration * 4)) {
+            instance->lfo_duration0 = instance->ramp_duration;
+            instance->lfo_duration1 = period - instance->ramp_duration;
+        } else {
+            instance->lfo_duration1 = period * 3 / 4;
+            instance->lfo_duration0 = period - instance->lfo_duration1;
+        }
+        instance->lfo_increment0 = FP_SIZE / (int32_t)instance->lfo_duration0;
+        instance->lfo_increment1 = -FP_SIZE / (int32_t)instance->lfo_duration1;
+        instance->lfo_duration = instance->lfo_duration1;
+        instance->lfo_increment = instance->lfo_increment1;
+        break;
+      case 3:  /* square */
+        instance->lfo_phase = 0;
+        instance->lfo_value = FP_SIZE;
+        if (period >= (instance->ramp_duration * 6)) {
+            instance->lfo_duration0 = (period / 2) - instance->ramp_duration;
+            instance->lfo_duration1 = instance->ramp_duration;
+        } else {
+            instance->lfo_duration0 = period / 3;
+            instance->lfo_duration1 = (period / 2) - instance->lfo_duration0;
+        }
+        instance->lfo_increment1 = FP_SIZE / (int32_t)instance->lfo_duration1;
+        instance->lfo_increment0 = -instance->lfo_increment1;
+        instance->lfo_duration = instance->lfo_duration0;
+        instance->lfo_increment = 0;
+        break;
+      case 4:  /* sine */
+        instance->lfo_value = FP_SIZE / 4; /* phase of pi/2 in cosine table */
+        instance->lfo_increment = FP_SIZE / period;
+        break;
+      case 5:  /* sample/hold */
+        instance->lfo_phase = 0;
+        instance->lfo_value = rand() & FP_MASK;
+        if (period >= (instance->ramp_duration * 4)) {
+            instance->lfo_duration0 = period - instance->ramp_duration;
+            instance->lfo_duration1 = instance->ramp_duration;
+        } else {
+            instance->lfo_duration0 = period * 3 / 4;
+            instance->lfo_duration1 = period - instance->lfo_duration0;
+        }
+        instance->lfo_duration = instance->lfo_duration0;
+        instance->lfo_increment = 0;
+        break;
+    }
+}
+
+/*
+ * dx7_lfo_reset
+ *
+ * called from hexter_activate() to give instance LFO parameters sane values
+ * until they're set by a playing voice
+ */
+void
+dx7_lfo_reset(hexter_instance_t *instance)
+{
+    instance->lfo_speed = 20;
+    instance->lfo_wave = 1;
+    instance->lfo_delay = 255;  /* force setup at first note on */
+    instance->lfo_value_for_pitch = 0.0;
+    dx7_lfo_set_speed(instance);
+}
+
+void
+dx7_lfo_set(hexter_instance_t *instance, dx7_voice_t *voice)
+{
+    int set_speed = 0;
+
+    instance->lfo_wave = voice->lfo_wave;
+    if (instance->lfo_speed != voice->lfo_speed) {
+        instance->lfo_speed = voice->lfo_speed;
+        set_speed = 1;
+    }
+    if (voice->lfo_key_sync) {
+        set_speed = 1; /* because we need to reset the LFO phase */
+    }
+    if (set_speed)
+        dx7_lfo_set_speed(instance);
+    if (instance->lfo_delay != voice->lfo_delay) {
+        instance->lfo_delay = voice->lfo_delay;
+        if (voice->lfo_delay > 0) {
+            instance->lfo_delay_value[0] = 0;
+            /* -FIX- Jamie's early approximation, replace when he has more data */
+            instance->lfo_delay_duration[0] =
+                lrintf(instance->sample_rate *
+                       (0.00175338f * pow((float)voice->lfo_delay, 3.10454f) + 169.344f - 168.0f) /
+                       1000.0f);
+            instance->lfo_delay_increment[0] = 0;
+            instance->lfo_delay_value[1] = 0;
+            /* -FIX- Jamie's early approximation, replace when he has more data */
+            instance->lfo_delay_duration[1] =
+                lrintf(instance->sample_rate *
+                       (0.321877f * pow((float)voice->lfo_delay, 2.01163) + 494.201f - 168.0f) /
+                       1000.0f);                                                 /* time from note-on until full on */
+            instance->lfo_delay_duration[1] -= instance->lfo_delay_duration[0];  /* now time from end-of-delay until full */
+            instance->lfo_delay_increment[1] = FP_SIZE / (int32_t)instance->lfo_delay_duration[1];
+            instance->lfo_delay_value[2] = FP_SIZE;
+            instance->lfo_delay_duration[2] = 0;
+            instance->lfo_delay_increment[2] = 0;
+        } else {
+            instance->lfo_delay_value[0] = FP_SIZE;
+            instance->lfo_delay_duration[0] = 0;
+            instance->lfo_delay_increment[0] = 0;
+        }
+        /* -FIX- The TX7 resets the lfo delay for all playing notes at each
+         * new note on. We're not doing that yet, and I don't really wanna,
+         * 'cause it's stupid.... */
+    }
+}
+
+void
+dx7_lfo_update(hexter_instance_t *instance, unsigned long sample_count)
+{
+    unsigned long sample;
+
+    switch (instance->lfo_wave) {
+      default:
+      case 0:  /* triangle */
+      case 1:  /* saw down */
+      case 2:  /* saw up */
+        for (sample = 0; sample < sample_count; sample++) {
+            instance->lfo_buffer[sample] = instance->lfo_value;
+            instance->lfo_value += instance->lfo_increment;
+            if (!(--instance->lfo_duration)) {
+                if (instance->lfo_phase) {
+                    instance->lfo_phase = 0;
+                    instance->lfo_value = 0;
+                    instance->lfo_duration = instance->lfo_duration0;
+                    instance->lfo_increment = instance->lfo_increment0;
+                } else {
+                    instance->lfo_phase = 1;
+                    instance->lfo_value = FP_SIZE;
+                    instance->lfo_duration = instance->lfo_duration1;
+                    instance->lfo_increment = instance->lfo_increment1;
+                }
+            }
+        }
+        instance->lfo_value_for_pitch = FP_TO_DOUBLE(instance->lfo_value) * 2.0 - 1.0;  /* -FIX- this is still ramped for saw! */
+        break;
+      case 3:  /* square */
+        for (sample = 0; sample < sample_count; sample++) {
+            instance->lfo_buffer[sample] = instance->lfo_value;
+            instance->lfo_value += instance->lfo_increment;
+            if (!(--instance->lfo_duration)) {
+                switch (instance->lfo_phase) {
+                  default:
+                  case 0:
+                    instance->lfo_phase = 1;
+                    instance->lfo_duration = instance->lfo_duration1;
+                    instance->lfo_increment = instance->lfo_increment0;
+                    break;
+                  case 1:
+                    instance->lfo_phase = 2;
+                    instance->lfo_value = 0;
+                    instance->lfo_duration = instance->lfo_duration0;
+                    instance->lfo_increment = 0;
+                    break;
+                  case 2:
+                    instance->lfo_phase = 3;
+                    instance->lfo_duration = instance->lfo_duration1;
+                    instance->lfo_increment = instance->lfo_increment1;
+                    break;
+                  case 3:
+                    instance->lfo_phase = 0;
+                    instance->lfo_value = FP_SIZE;
+                    instance->lfo_duration = instance->lfo_duration0;
+                    instance->lfo_increment = 0;
+                    break;
+                }
+            }
+        }
+        if (instance->lfo_phase == 0 || instance->lfo_phase == 3)
+            instance->lfo_value_for_pitch = 1.0;
+        else
+            instance->lfo_value_for_pitch = -1.0;
+        break;
+      case 4:  /* sine */
+        for (sample = 0; sample < sample_count; sample++) {
+            int32_t phase, index, out;
+
+            phase = instance->lfo_value;
+            index = (phase >> FP_TO_SINE_SHIFT) & SINE_MASK;
+            out = dx7_voice_sin_table[index];
+            out += (((int64_t)(dx7_voice_sin_table[index + 1] - out) *
+                     (int64_t)(phase & FP_TO_SINE_MASK)) >>
+                    (FP_SHIFT + FP_TO_SINE_SHIFT));
+            out = (out + FP_SIZE) >> 1;  /* shift to unipolar */
+            instance->lfo_buffer[sample] = out;
+            instance->lfo_value += instance->lfo_increment;
+        }
+        instance->lfo_value_for_pitch = FP_TO_DOUBLE(instance->lfo_buffer[sample - 1]) * 2.0 - 1.0;
+        break;
+      case 5:  /* sample/hold */
+        for (sample = 0; sample < sample_count; sample++) {
+            instance->lfo_buffer[sample] = instance->lfo_value;
+            instance->lfo_value += instance->lfo_increment;
+            if (!(--instance->lfo_duration)) {
+                if (instance->lfo_phase) {
+                    instance->lfo_phase = 0;
+                    instance->lfo_value = instance->lfo_target;
+                    instance->lfo_duration = instance->lfo_duration0;
+                    instance->lfo_increment = 0;
+                } else {
+                    instance->lfo_phase = 1;
+                    instance->lfo_duration = instance->lfo_duration1;
+                    instance->lfo_target = rand() & FP_MASK;
+                    instance->lfo_increment = (instance->lfo_target - instance->lfo_value) /
+                                                  (int32_t)instance->lfo_duration;
+                }
+            }
+        }
+        instance->lfo_value_for_pitch = FP_TO_DOUBLE(instance->lfo_target) * 2.0 - 1.0;
+        break;
+    }
+}
+
+/* ===== modulation functions ===== */
+
+/* there used to be a dx7_voice_update_pitch_bend() here, but it didn't
+ * have anything to do.... */
+
+void
+dx7_voice_update_mod_depths(hexter_instance_t *instance, dx7_voice_t* voice)
+{
+    unsigned char kp = instance->key_pressure[voice->key];
+    unsigned char cp = instance->channel_pressure;
+    float pressure;
+    float pdepth, adepth, mdepth, edepth;
+
+    /* add the channel and key pressures together in a way that 'feels' good */
+    if (kp > cp) {
+        pressure = (float)kp / 127.0f;
+        pressure += (1.0f - pressure) * ((float)cp / 127.0f);
+    } else {
+        pressure = (float)cp / 127.0f;
+        pressure += (1.0f - pressure) * ((float)kp / 127.0f);
+    }
+
+    /* calculate modulation depths */
+    pdepth = (float)voice->lfo_pmd / 99.0f;
+    voice->pitch_mod_depth_pmd = (double)dx7_voice_pms_to_semitones[voice->lfo_pms] *
+                                 (double)pdepth;
+    // -FIX- this could be optimized:
+    // -FIX- this just adds everything together -- maybe it should limit the result, or
+    // combine the various mods like update_pressure() does
+    pdepth = (instance->mod_wheel_assign & 0x01 ?
+                 // -FIX- this assumes that mod_wheel_sensitivity, etc. scale linearly => verify
+                 (float)instance->mod_wheel_sensitivity / 15.0f * instance->mod_wheel :
+                 0.0f) +
+             (instance->foot_assign & 0x01 ?
+                 (float)instance->foot_sensitivity / 15.0f * instance->foot :
+                 0.0f) +
+             (instance->pressure_assign & 0x01 ?
+                 (float)instance->pressure_sensitivity / 15.0f * pressure :
+                 0.0f) +
+             (instance->breath_assign & 0x01 ?
+                 (float)instance->breath_sensitivity / 15.0f * instance->breath :
+                 0.0f);
+    voice->pitch_mod_depth_mods = (double)dx7_voice_pms_to_semitones[voice->lfo_pms] *
+                                  (double)pdepth;
+
+    // -FIX- these are total guesses at how to combine/limit the amp mods:
+    adepth = dx7_voice_amd_to_ol_adjustment[voice->lfo_amd];
+    // -FIX- this could be optimized:
+    mdepth = (instance->mod_wheel_assign & 0x02 ?
+                 dx7_voice_mss_to_ol_adjustment[instance->mod_wheel_sensitivity] *
+                     instance->mod_wheel :
+                 0.0f) +
+             (instance->foot_assign & 0x02 ?
+                 dx7_voice_mss_to_ol_adjustment[instance->foot_sensitivity] *
+                     instance->foot :
+                 0.0f) +
+             (instance->pressure_assign & 0x02 ?
+                 dx7_voice_mss_to_ol_adjustment[instance->pressure_sensitivity] *
+                     pressure :
+                 0.0f) +
+             (instance->breath_assign & 0x02 ?
+                 dx7_voice_mss_to_ol_adjustment[instance->breath_sensitivity] *
+                     instance->breath :
+                 0.0f);
+    edepth = // -FIX- this could be optimized:
+             (instance->mod_wheel_assign & 0x04 ?
+                 dx7_voice_mss_to_ol_adjustment[instance->mod_wheel_sensitivity] *
+                     (1.0f - instance->mod_wheel) :
+                 0.0f) +
+             (instance->foot_assign & 0x04 ?
+                 dx7_voice_mss_to_ol_adjustment[instance->foot_sensitivity] *
+                     (1.0f - instance->foot) :
+                 0.0f) +
+             (instance->pressure_assign & 0x04 ?
+                 dx7_voice_mss_to_ol_adjustment[instance->pressure_sensitivity] *
+                     (1.0f - pressure) :
+                 0.0f) +
+             (instance->breath_assign & 0x04 ?
+                 dx7_voice_mss_to_ol_adjustment[instance->breath_sensitivity] *
+                     (1.0f - instance->breath) :
+                 0.0f);
+
+    /* full-scale amp mod for adepth and edepth should be 52.75 and
+     * their sum _must_ be limited to less than 128, or bad things will happen! */
+    if (adepth > 127.5f) adepth = 127.5f;
+    if (adepth + mdepth > 127.5f)
+        mdepth = 127.5f - adepth;
+    if (adepth + mdepth + edepth > 127.5f)
+        edepth = 127.5f - (adepth + mdepth);
+
+    voice->amp_mod_lfo_amd_target = FLOAT_TO_FP(adepth);
+    if (voice->amp_mod_lfo_amd_value <= INT_TO_FP(-64)) {
+        voice->amp_mod_lfo_amd_value = voice->amp_mod_lfo_amd_target;
+        voice->amp_mod_lfo_amd_increment = 0;
+        voice->amp_mod_lfo_amd_duration = 0;
+    } else {
+        voice->amp_mod_lfo_amd_duration = instance->ramp_duration;
+        voice->amp_mod_lfo_amd_increment = (voice->amp_mod_lfo_amd_target - voice->amp_mod_lfo_amd_value) /
+                                           (int32_t)voice->amp_mod_lfo_amd_duration;
+    }
+    voice->amp_mod_lfo_mods_target = FLOAT_TO_FP(mdepth);
+    if (voice->amp_mod_lfo_mods_value <= INT_TO_FP(-64)) {
+        voice->amp_mod_lfo_mods_value = voice->amp_mod_lfo_mods_target;
+        voice->amp_mod_lfo_mods_increment = 0;
+        voice->amp_mod_lfo_mods_duration = 0;
+    } else {
+        voice->amp_mod_lfo_mods_duration = instance->ramp_duration;
+        voice->amp_mod_lfo_mods_increment = (voice->amp_mod_lfo_mods_target - voice->amp_mod_lfo_mods_value) /
+                                           (int32_t)voice->amp_mod_lfo_mods_duration;
+    }
+    voice->amp_mod_env_target = FLOAT_TO_FP(edepth);
+    if (voice->amp_mod_env_value <= INT_TO_FP(-64)) {
+        voice->amp_mod_env_value = voice->amp_mod_env_target;
+        voice->amp_mod_env_increment = 0;
+        voice->amp_mod_env_duration = 0;
+    } else {
+        voice->amp_mod_env_duration = instance->ramp_duration;
+        voice->amp_mod_env_increment = (voice->amp_mod_env_target - voice->amp_mod_env_value) /
+                                           (int32_t)voice->amp_mod_env_duration;
     }
 }
 
@@ -751,10 +1179,18 @@ dx7_voice_calculate_runtime_parameters(hexter_instance_t *instance, dx7_voice_t*
     double freq;
 
     dx7_pitch_envelope_prepare(instance, voice);
-
+    voice->amp_mod_lfo_amd_value = INT_TO_FP(-64);   /* force initial setup */
+    voice->amp_mod_lfo_mods_value = INT_TO_FP(-64);
+    voice->amp_mod_env_value = INT_TO_FP(-64);
+    voice->lfo_delay_segment = 0;
+    voice->lfo_delay_value     = instance->lfo_delay_value[0];
+    voice->lfo_delay_duration  = instance->lfo_delay_duration[0];
+    voice->lfo_delay_increment = instance->lfo_delay_increment[0];
+    voice->mods_serial = instance->mods_serial - 1;  /* force mod depths update */
+    dx7_portamento_prepare(instance, voice);
     freq = dx7_voice_recalculate_frequency(instance, voice);
 
-    voice->volume_value = -1.0f;  /* force initial setup */
+    voice->volume_value = -1.0f;                     /* force initial setup */
     dx7_voice_recalculate_volume(instance, voice);
 
     for (i = 0; i < MAX_DX7_OPERATORS; i++) {
@@ -776,10 +1212,12 @@ void
 dx7_voice_setup_note(hexter_instance_t *instance, dx7_voice_t *voice)
 {
     dx7_voice_set_data(instance, voice);
+    hexter_instance_set_performance_data(instance);
+    dx7_lfo_set(instance, voice);
     dx7_voice_calculate_runtime_parameters(instance, voice);
 }
 
-static int
+static inline int
 limit(int x, int min, int max)
 {
     if (x < min) return min;
@@ -794,6 +1232,7 @@ void
 dx7_voice_set_data(hexter_instance_t *instance, dx7_voice_t *voice)
 {
     uint8_t *edit_buffer = instance->current_patch_buffer;
+    int compat059 = (instance->performance_buffer[0] & 0x01);  /* 0.5.9 compatibility */
     int i, j;
     double aux_feedbk;
 
@@ -813,6 +1252,7 @@ dx7_voice_set_data(hexter_instance_t *instance, dx7_voice_t *voice)
         voice->op[i].level_scaling_l_curve = eb_op[11] & 0x03;
         voice->op[i].level_scaling_r_curve = eb_op[12] & 0x03;
         voice->op[i].rate_scaling          = eb_op[13] & 0x07;
+        voice->op[i].amp_mod_sens          = (compat059 ? 0 : eb_op[14] & 0x03);
         voice->op[i].velocity_sens         = eb_op[15] & 0x07;
 
         for (j = 0; j < 4; j++) {
@@ -828,7 +1268,7 @@ dx7_voice_set_data(hexter_instance_t *instance, dx7_voice_t *voice)
 
     voice->algorithm = edit_buffer[134] & 0x1f;
 
-    aux_feedbk = (double)(edit_buffer[135] & 0x07) / (2.0 * M_PI) * 0.18 /* !FIX! feedback_scaling[voice->algorithm] */;
+    aux_feedbk = (double)(edit_buffer[135] & 0x07) / (2.0 * M_PI) * 0.18 /* -FIX- feedback_scaling[voice->algorithm] */;
 
     /* the "99.0" here is because we're also using this multiplier to scale the
      * eg level from 0-99 to 0-1 */
@@ -836,42 +1276,14 @@ dx7_voice_set_data(hexter_instance_t *instance, dx7_voice_t *voice)
 
     voice->osc_key_sync = edit_buffer[136] & 0x01;
 
+    voice->lfo_speed    = limit(edit_buffer[137], 0, 99);
+    voice->lfo_delay    = limit(edit_buffer[138], 0, 99);
+    voice->lfo_pmd      = limit(edit_buffer[139], 0, 99);
+    voice->lfo_amd      = limit(edit_buffer[140], 0, 99);
+    voice->lfo_key_sync = edit_buffer[141] & 0x01;
+    voice->lfo_wave     = limit(edit_buffer[142], 0, 5);
+    voice->lfo_pms      = (compat059 ? 0 : edit_buffer[143] & 0x07);
+
     voice->transpose = limit(edit_buffer[144], 0, 48);
-
-    /* !FIX! this doesn't get:
-     * per operator: 14 AMS
-     * 137-143 LFO parameters
-     */
-}
-
-/*
- * dx7_voice_update_pressure_mod
- */
-void
-dx7_voice_update_pressure_mod(hexter_instance_t *instance, dx7_voice_t *voice)
-{
-    unsigned char kp = instance->key_pressure[voice->key];
-    unsigned char cp = instance->channel_pressure;
-    float p;
-
-    /* add the channel and key pressures together in a way that 'feels' good */
-    if (kp > cp) {
-        p = (float)kp / 127.0f;
-        p += (1.0f - p) * ((float)cp / 127.0f);
-    } else {
-        p = (float)cp / 127.0f;
-        p += (1.0f - p) * ((float)kp / 127.0f);
-    }
-    /* !FIX! apply pressure modulation: */
-    /* voice->pressure = f(p); */
-}
-
-/*
- * dx7_voice_update_pitch_bend
- */
-void
-dx7_voice_update_pitch_bend(hexter_instance_t *instance, dx7_voice_t *voice)
-{
-    /* nothing to do here, it's automatic */
 }
 
