@@ -1,12 +1,14 @@
 /* hexter DSSI software synthesizer plugin
  *
- * Copyright (C) 2004 Sean Bolton and others.
+ * Copyright (C) 2004, 2006 Sean Bolton and others.
  *
  * Portions of this file may have come from Juan Linietsky's
  * rx-saturno, copyright (C) 2002 by Juan Linietsky.
  * Portions of this file may have come from Jeff Harrington's
  * dx72csound package, for which I've found no copyright
  * attribution.
+ * Significant reverse-engineering contributions were made by
+ * Jamie Bullock.
  * Other parts may have been inspired by Chowning/Bristow,
  * Pinkston, Yala Abdullah's website, Godric Wilkie' website;
  * see the AUTHORS file for more detail.
@@ -38,6 +40,12 @@
 static int dx7_voice_tables_initialized = 0;
 int32_t    dx7_voice_sin_table[SINE_SIZE + 1];
 
+extern int32_t dx7_voice_eg_ol_to_mod_index_table[257]; /* forward */
+extern int32_t dx7_voice_eg_ol_to_amp_table[257];       /* forward */
+
+int32_t       *dx7_voice_eg_ol_to_mod_index = &dx7_voice_eg_ol_to_mod_index_table[128];
+int32_t       *dx7_voice_eg_ol_to_amp       = &dx7_voice_eg_ol_to_amp_table[128];
+
 void dx7_voice_init_tables(void) {
 
     int i;
@@ -56,9 +64,9 @@ void dx7_voice_init_tables(void) {
 #if FP_SHIFT != 24
         /* The fixed-point tables below are all in s7.24 format.  Shift
          * them to match FP_SHIFT. */
-        for (i = 0; i <= 128; i++) {
-            dx7_voice_eg_ol_to_mod_index[i] >>= (24 - FP_SHIFT);
-            dx7_voice_eg_ol_to_amp[i]       >>= (24 - FP_SHIFT);
+        for (i = 0; i <= 256; i++) {
+            dx7_voice_eg_ol_to_mod_index_table[i] >>= (24 - FP_SHIFT);
+            dx7_voice_eg_ol_to_amp_table[i]       >>= (24 - FP_SHIFT);
         }
 #endif
 
@@ -237,9 +245,11 @@ float dx7_voice_eg_rate_rise_percent[128] = {  /* checked, matches P/H/Op f05 */
 
 /* This table converts an output level of 0 to 99 into a phase modulation
  * index of 0 to ~2.089 periods, expressed in s7.24 fixed point.  It
- * actually extends beyond 99, since velocities above 100 can produce
- * output levels above 99, plus it includes a 129th 'guard' point.
- * I believe it is based on information from the Chowning/Bristow
+ * actually extends below 0 and beyond 99, since amplitude modulation can
+ * produce 'negative' output levels, and velocities above 100 can produce
+ * output levels above 99, plus it includes a 257th 'guard' point.
+ * Table index 128 corresponds to output level 0, and index 227 to OL 99.
+ * I believe this is based on information from the Chowning/Bristow
  * book (see the CREDITS file), filtered down to me through the work of
  * Pinkston, Harrington, and Abdullah as I found it on the Internet.  The
  * code used to calculate it looks something like this:
@@ -270,8 +280,16 @@ float dx7_voice_eg_rate_rise_percent[128] = {  /* checked, matches P/H/Op f05 */
  *        printf(" %6d,", DOUBLE_TO_FP(mi));
  *    }
  */
-int32_t dx7_voice_eg_ol_to_mod_index[129] = {
+int32_t dx7_voice_eg_ol_to_mod_index_table[257] = {
 
+    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
     0, 659, 1319, 1978, 2638, 3298, 4277, 5086, 6049, 7193, 8554,
     10173, 12098, 14387, 17109, 20346, 22188, 24196, 28774, 31378,
     37315, 40693, 44376, 48392, 52772, 57548, 62757, 68437, 74631,
@@ -297,9 +315,17 @@ int32_t dx7_voice_eg_ol_to_mod_index[129] = {
 /* This table converts an output level of 0 to 99 into an output amplitude
  * scale factor of 0 to 1 in s7.24 fixed point.  It converts a linear 0-99
  * envelope to an exponential volume, and is just a normalized version of
- * dx7_voice_eg_ol_to_mod_index[] (see its description above). */
-int32_t dx7_voice_eg_ol_to_amp[129] = {
+ * dx7_voice_eg_ol_to_mod_index_table[] (see its description above). */
+int32_t dx7_voice_eg_ol_to_amp_table[257] = {
 
+    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
     0, 315, 631, 947, 1263, 1579, 2048, 2435, 2896, 3444, 4096,
     4870, 5792, 6888, 8192, 9741, 10623, 11585, 13777, 15024, 17866,
     19483, 21247, 23170, 25267, 27554, 30048, 32768, 35733, 38967,
@@ -393,3 +419,85 @@ double dx7_voice_pitch_level_to_shift[128] = {
 
 };
 
+/* This table converts LFO speed to frequency in Hz. It is based on
+ * interpolation of Jamie Bullock's measurements. */
+float dx7_voice_lfo_frequency[128] = {
+     0.062506,  0.124815,  0.311474,  0.435381,  0.619784,
+     0.744396,  0.930495,  1.116390,  1.284220,  1.496880,
+     1.567830,  1.738994,  1.910158,  2.081322,  2.252486,
+     2.423650,  2.580668,  2.737686,  2.894704,  3.051722,
+     3.208740,  3.366820,  3.524900,  3.682980,  3.841060,
+     3.999140,  4.159420,  4.319700,  4.479980,  4.640260,
+     4.800540,  4.953584,  5.106628,  5.259672,  5.412716,
+     5.565760,  5.724918,  5.884076,  6.043234,  6.202392,
+     6.361550,  6.520044,  6.678538,  6.837032,  6.995526,
+     7.154020,  7.300500,  7.446980,  7.593460,  7.739940,
+     7.886420,  8.020588,  8.154756,  8.288924,  8.423092,
+     8.557260,  8.712624,  8.867988,  9.023352,  9.178716,
+     9.334080,  9.669644, 10.005208, 10.340772, 10.676336,
+    11.011900, 11.963680, 12.915460, 13.867240, 14.819020,
+    15.770800, 16.640240, 17.509680, 18.379120, 19.248560,
+    20.118000, 21.040700, 21.963400, 22.886100, 23.808800,
+    24.731500, 25.759740, 26.787980, 27.816220, 28.844460,
+    29.872700, 31.228200, 32.583700, 33.939200, 35.294700,
+    36.650200, 37.812480, 38.974760, 40.137040, 41.299320,
+    42.461600, 43.639800, 44.818000, 45.996200, 47.174400,
+    47.174400, 47.174400, 47.174400, 47.174400, 47.174400,
+    47.174400, 47.174400, 47.174400, 47.174400, 47.174400,
+    47.174400, 47.174400, 47.174400, 47.174400, 47.174400,
+    47.174400, 47.174400, 47.174400, 47.174400, 47.174400,
+    47.174400, 47.174400, 47.174400, 47.174400, 47.174400,
+    47.174400, 47.174400, 47.174400
+};
+
+/* This table converts pitch modulation sensitivity to semitones at full
+ * modulation (assuming a perfectly linear pitch mod depth to pitch
+ * relationship).  It is from a simple averaging of Jamie Bullock's
+ * TX-data-1/PMD and TX-data-2/ENV data, and ignores the apparent ~0.1
+ * semitone positive bias that Jamie observed. [-FIX- smbolton: my
+ * inclination would be to call this bias, if it's reproducible, a
+ * non-desirable 'bug', and _not_ implement it in hexter. And, at
+ * least for my own personal build, I'd change that PMS=7 value to a
+ * full octave, since that's one thing that's always bugged me about
+ * my TX7.  Thoughts? ] */
+float dx7_voice_pms_to_semitones[8] = {
+    0.0, 0.450584, 0.900392, 1.474744,
+    2.587385, 4.232292, 6.982097, /* 11.722111 */ 12.0
+};
+
+/* This table converts amplitude modulation depth to output level
+ * reduction at full modulation with an amplitude modulation sensitivity
+ * of 3.  It was constructed from regression of a very few data points,
+ * using this code:
+ *   perl -e 'for ($i = 0; $i <= 99; $i++) { printf " %f,\n", exp($i * 0.0428993 - 0.285189); }' >x.c
+ * and is probably rather rough in its accuracy. -FIX- */
+float dx7_voice_amd_to_ol_adjustment[100] = {
+    0.0, 0.784829, 0.819230, 0.855139, 0.892622, 0.931748,
+    0.972589, 1.015221, 1.059721, 1.106171, 1.154658, 1.205270,
+    1.258100, 1.313246, 1.370809, 1.430896, 1.493616, 1.559085,
+    1.627424, 1.698759, 1.773220, 1.850945, 1.932077, 2.016765,
+    2.105166, 2.197441, 2.293761, 2.394303, 2.499252, 2.608801,
+    2.723152, 2.842515, 2.967111, 3.097167, 3.232925, 3.374633,
+    3.522552, 3.676956, 3.838127, 4.006362, 4.181972, 4.365280,
+    4.556622, 4.756352, 4.964836, 5.182458, 5.409620, 5.646738,
+    5.894251, 6.152612, 6.422298, 6.703805, 6.997652, 7.304378,
+    7.624549, 7.958754, 8.307609, 8.671754, 9.051861, 9.448629,
+    9.862789, 10.295103, 10.746365, 11.217408, 11.709099,
+    12.222341, 12.758080, 13.317302, 13.901036, 14.510357,
+    15.146387, 15.810295, 16.503304, 17.226690, 17.981783,
+    18.769975, 19.592715, 20.451518, 21.347965, 22.283705,
+    23.260462, 24.280032, 25.344294, 26.455204, 27.614809,
+    28.825243, 30.088734, 31.407606, 32.784289, 34.221315,
+    35.721330, 37.287095, 38.921492, 40.627529, 42.408347,
+    44.267222, 46.207578, 48.232984, 50.347169, 52.75
+};
+
+/* This table converts modulation source sensitivity (e.g. 'foot
+ * controller sensitivity') into output level reduction at full modulation
+ * with amplitude modulation sensitivity 3.  It's basically just the above
+ * table scaled for 0 to 15 instead of 0 to 99. */
+float dx7_voice_mss_to_ol_adjustment[16] = {
+    0.0, 0.997948, 1.324562, 1.758071, 2.333461, 3.097167, 4.110823,
+    5.456233, 7.241976, 9.612164, 12.758080, 16.933606, 22.475719,
+    29.831681, 39.595137, 52.75
+};
