@@ -1,6 +1,6 @@
 /* hexter DSSI software synthesizer plugin
  *
- * Copyright (C) 2004 Sean Bolton and others.
+ * Copyright (C) 2004, 2006 Sean Bolton and others.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -25,17 +25,19 @@
 
 #define DX7_VOICE_SIZE_PACKED     128
 #define DX7_VOICE_SIZE_UNPACKED   155
+#define DX7_PERFORMANCE_SIZE       64
 #define DX7_DUMP_SIZE_SINGLE      155+8
 #define DX7_DUMP_SIZE_BULK        4096+8
 
 typedef struct _hexter_synth_t    hexter_synth_t;
 typedef struct _hexter_instance_t hexter_instance_t;
 
-typedef struct _dx7_patch_t    dx7_patch_t;
-typedef struct _dx7_voice_t    dx7_voice_t;
-typedef struct _dx7_op_eg_t    dx7_op_eg_t;
-typedef struct _dx7_pitch_eg_t dx7_pitch_eg_t;
-typedef struct _dx7_op_t       dx7_op_t;
+typedef struct _dx7_patch_t       dx7_patch_t;
+typedef struct _dx7_voice_t       dx7_voice_t;
+typedef struct _dx7_op_eg_t       dx7_op_eg_t;
+typedef struct _dx7_pitch_eg_t    dx7_pitch_eg_t;
+typedef struct _dx7_portamento_t  dx7_portamento_t;
+typedef struct _dx7_op_t          dx7_op_t;
 
 /* in hexter.c: */
 int   dssp_voicelist_mutex_lock(void);
